package com.cfp.jcperez.electricost;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ListView;
import android.widget.Toast;

import com.jjoe64.graphview.GraphView;
import com.jjoe64.graphview.series.DataPoint;
import com.jjoe64.graphview.series.LineGraphSeries;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.GregorianCalendar;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

/**
 * Created by Juan Carlos Perez Gonzalez on 10/11/2015.
 * Actividad que obtiene el listado de los precios de la luz y muestra dicho listado y una gráfica con los mismos
 */
public class ObtenerPreciosActivity extends ActionBarActivity {

    private ArrayList<String> arrHoras = new  ArrayList<String>();
    private ArrayList<Double> arrTarifa = new  ArrayList<Double>();
    private String tipo="";
   @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.listadoprecios);
        //inicializamos la vista de la grafica
        GraphView graph = (GraphView) findViewById(R.id.grafica);
        graph.getViewport().setXAxisBoundsManual(true);
        graph.getViewport().setMinX(0);
        graph.getViewport().setMaxX(24);

        graph.getViewport().setYAxisBoundsManual(true);
        graph.getGridLabelRenderer().setHorizontalLabelsVisible(true);
        graph.getGridLabelRenderer().setVerticalLabelsVisible(false);

        graph.setTitle("-");
        graph.setTitleColor(Color.BLACK);
        graph.setTitleTextSize(60);

        //recuperamos la fecha actual y obtenemos el precio para dicha fecha
        GregorianCalendar calendar = new GregorianCalendar();
        tipo =  getIntent().getExtras().getString("Tipo");
        //Log.d("JCP-----", "tipo " + tipo);
        String[] param= new String[2];
        param[0]=tipo;
        param[1]=(new SimpleDateFormat("yyyyMMdd")).format(calendar.getTime());
        //Log.d("JCP--------", "onCreate");
        new AccesoWebREE().execute(param);
   }

    @Override protected void onPause() {
        super.onPause();
        Log.d("JCP--------", "onPause");
    }

    @Override protected void onResume(){
        super.onResume();
        Log.d("JCP--------", "onResume");
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_listado, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            lanzarPreferencias(null);
        }
        if (id == R.id.acercaDe) {
            lanzarAcercaDe(null);
        }

        if (id == R.id.Calendario) {
            lanzarCalendario(null);
        }
        return super.onOptionsItemSelected(item);
    }

    public void lanzarAcercaDe(View view){
        Intent i = new Intent (this, AcercaDe.class);
        startActivity(i);
    }

    public void lanzarPreferencias(View view){
        Intent i = new Intent (this, Preferencias.class);
        startActivity(i);
    }

    @TargetApi(11)
    public void lanzarCalendario(View view){
        Intent i = new Intent (this, CalendarioActivity.class);
        startActivityForResult(i, 111);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch(requestCode) {
            case (111) : {
                if (resultCode == Activity.RESULT_OK) {
                    String[] param= new String[2];
                    param[0]=tipo;
                    param[1]=data.getStringExtra("Fecha");
                    ListView listaPr = (ListView)findViewById(R.id.listaPrecios);
                    arrHoras.clear();
                    arrTarifa=null;
                    listaPr.setAdapter(null);
                    GraphView graph = (GraphView) findViewById(R.id.grafica);
                    graph.removeAllSeries();
                    new AccesoWebREE().execute(param);
                }
                break;
            }
        }
    }

    //clase que se conecta a la web http://www.esios.ree.es/ de REE para obtener el listado de los precios de la luz
    private class AccesoWebREE extends AsyncTask<String, Void, String> {
        private ProgressDialog progreso;
        private String fecha;
        private String tituloGraf;
        @Override protected void onPreExecute() {
            progreso = new ProgressDialog(ObtenerPreciosActivity.this);
            progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            progreso.setMessage(getResources().getString(R.string.Obteniendodatos));
            progreso.setCancelable(false);
            progreso.show();
        }
        @Override protected String doInBackground(String... param) {
            try {
                fecha = param[1];
                if (tipo.equals("BtnTG")) {
                    tituloGraf=getString(R.string.tarifageneral);
                }
                if (tipo.equals("BtnTN")) {
                    tituloGraf=getString(R.string.tarifanocturna);
                }
                if (tipo.equals("BtnCE")) {
                    tituloGraf=getString(R.string.CocheElectrico);
                }
                ConnectivityManager cm =(ConnectivityManager)getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
                if (cm!=null) {
                    NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
                    if (activeNetwork != null && activeNetwork.isConnected()) {
                        String tipo = param[0];
                        if (tipo == null) {
                            return null;
                        }
                        URL url = new URL("http://www.esios.ree.es/Solicitar/PVPC_DD_" + param[1] + ".xls&es");
                        //Log.d("JCP-----", "URL " + url.toString());
                        HttpURLConnection conexion = (HttpURLConnection) url.openConnection();
                        Log.d("JCP-----", "Conexion ");
                        if (conexion.getResponseCode() == HttpURLConnection.HTTP_OK) {
                            SAXParserFactory fabrica = SAXParserFactory.newInstance();
                            SAXParser parser = fabrica.newSAXParser();
                            XMLReader lector = parser.getXMLReader();
                            ManejadorXMLREE manejadorXMLREE = new ManejadorXMLREE();
                            lector.setContentHandler(manejadorXMLREE);
                            InputSource in = new InputSource(conexion.getInputStream());
                            in.setEncoding("ISO-8859-1");
                            lector.parse(in);
                            arrHoras = manejadorXMLREE.getArrHora();

                            if (tipo.equals("BtnTG")) {
                                arrTarifa = manejadorXMLREE.getArrTarifaGeneral();
                            }
                            if (tipo.equals("BtnTN")) {
                                arrTarifa = manejadorXMLREE.getArrTarifaNocturna();
                            }
                            if (tipo.equals("BtnCE")) {
                                arrTarifa = manejadorXMLREE.getArrTarifaVehiculo();
                            }
                        }
                        conexion.disconnect();
                        if (arrTarifa.size() > 24) {
                            arrTarifa.remove(24);
                        }
                        if (arrHoras.size() > 24) {
                            arrHoras.remove(24);
                        }

                        if (arrTarifa == null || arrTarifa.size() < 1) {
                            return "-1";
                        } else {
                            return "1";
                        }
                    }else{
                        return "-2";
                    }
                }else{
                    return "-2";
                }
            }catch (Exception e){
                e.printStackTrace();
                Log.d("JCP-----", "ERRORRRRRRRRRRR..." + e.toString());
                return "-1";
            }
        }
        @Override protected void onPostExecute(String res) {
            ListView lista = (ListView) findViewById(R.id.listaPrecios);
            fecha= fecha.substring(6)+"-"+fecha.substring(4,6)+"-"+fecha.substring(0,4);
            if ("-1".equals(res)) {
                Toast toast1 =
                        Toast.makeText(getApplicationContext(),
                                getString(R.string.nosehanencontradodatos) + fecha, Toast.LENGTH_LONG);

                toast1.show();
                GraphView graph = (GraphView) findViewById(R.id.grafica);
                graph.setTitle(tituloGraf + ": "+ fecha);
                graph.removeAllSeries();
            }else if("-2".equals(res)){
                Toast toast1 =   Toast.makeText(getApplicationContext(),
                        getString(R.string.nohayconexionalared), Toast.LENGTH_LONG);
                toast1.show();
                GraphView graph = (GraphView) findViewById(R.id.grafica);
                graph.setTitle(tituloGraf + ": "+ fecha);
                graph.removeAllSeries();
            }else {
                ListAdapter adapter = new ListAdapter(ObtenerPreciosActivity.this, arrHoras, arrTarifa, Collections.min(arrTarifa).doubleValue(), Collections.max(arrTarifa).doubleValue());
                lista.setAdapter(adapter);
                Toast toast1 =
                        Toast.makeText(getApplicationContext(),
                                getString(R.string.preciosparael) +" " +fecha, Toast.LENGTH_LONG);

                toast1.show();

                //dibujamos la grafica
                GraphView graph = (GraphView) findViewById(R.id.grafica);
                graph.setTitle(tituloGraf + ": "+ fecha);
                graph.getViewport().setMinY(Collections.min(arrTarifa).doubleValue());
                graph.getViewport().setMaxY(Collections.max(arrTarifa).doubleValue());
                DataPoint [] arrDP = new DataPoint[24];
                for(int i=0;i<arrTarifa.size() && i <24;i++){
                    if (i==0){
                        DataPoint dp = new DataPoint(0.3, arrTarifa.get(i));
                        arrDP[i] = dp;
                    }else if(i==23){
                        DataPoint dp = new DataPoint(23.3, arrTarifa.get(i));
                        arrDP[i] = dp;
                    }else {
                        DataPoint dp = new DataPoint(i, arrTarifa.get(i));
                        arrDP[i] = dp;
                    }
                }
                LineGraphSeries<DataPoint> series = new LineGraphSeries<DataPoint>(arrDP);
                series.setColor(Color.parseColor(getString(R.string.colorSerieGrafica)));
                graph.addSeries(series);
            }
            progreso.dismiss();
        }
        @Override protected void onCancelled() {
            progreso.dismiss();
        }
    }
}
